@@ -14,11 +14,12 @@
 #               6) http://forums.dpreview.com/forums/readflat.asp?forum=1012&thread=31350384
 #                  and http://forum.photome.de/viewtopic.php?f=2&t=353&p=742#p740
 #               7) Kai Lappalainen private communication
-#               8) http://u88.n24.queensu.ca/exiftool/forum/index.php/topic,5223.0.html
+#               8) https://exiftool.org/forum/index.php/topic,5223.0.html
 #               9) Zilvinas Brobliauskas private communication
 #               10) Albert Shan private communication
-#               11) http://u88.n24.queensu.ca/exiftool/forum/index.php/topic,8377.0.html
-#               12) http://u88.n24.queensu.ca/exiftool/forum/index.php/topic,9607.0.html
+#               11) https://exiftool.org/forum/index.php/topic,8377.0.html
+#               12) https://exiftool.org/forum/index.php/topic,9607.0.html
+#               13) https://exiftool.org/forum/index.php/topic=10481.0.html
 #               IB) Iliah Borg private communication (LibRaw)
 #               JD) Jens Duttke private communication
 #------------------------------------------------------------------------------
@@ -30,12 +31,13 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.67';
+$VERSION = '1.76';
 
 sub ProcessFujiDir($$$);
 sub ProcessFaceRec($$$);
 
 # the following RAF version numbers have been tested for writing:
+# (as of ExifTool 11.70, this lookup is no longer used if the version number is numerical)
 my %testedRAF = (
     '0100' => 'E550, E900, F770, S5600, S6000fd, S6500fd, HS10/HS11, HS30, S200EXR, X100, XF1, X-Pro1, X-S1, XQ2 Ver1.00, X-T100, GFX 50R, XF10',
     '0101' => 'X-E1, X20 Ver1.01, X-T3',
@@ -49,11 +51,14 @@ my %testedRAF = (
     '0144' => 'X100T Ver1.44',
     '0159' => 'S2Pro Ver1.00',
     '0200' => 'X10 Ver2.00',
+    '0201' => 'X-H1 Ver2.01',
     '0212' => 'S3Pro Ver2.12',
     '0216' => 'S3Pro Ver2.16', # (NC)
     '0218' => 'S3Pro Ver2.18',
+    '0240' => 'X-E1 Ver2.40',
     '0264' => 'F700 Ver2.00',
     '0266' => 'S9500 Ver1.01',
+    '0261' => 'X-E1 Ver2.61',
     '0269' => 'S9500 Ver1.02',
     '0271' => 'S3Pro Ver2.71', # UV/IR model?
     '0300' => 'X-E2',
@@ -260,10 +265,22 @@ my %faceCategories = (
             16 => 'Commander',
             0x8000 => 'Not Attached', #10 (X-T2) (or external flash off)
             0x8120 => 'TTL', #10 (X-T2)
+            0x8320 => 'TTL Auto - Did not fire',
             0x9840 => 'Manual', #10 (X-T2)
+            0x9860 => 'Flash Commander', #13
             0x9880 => 'Multi-flash', #10 (X-T2)
             0xa920 => '1st Curtain (front)', #10 (EF-X500 flash)
+            0xaa20 => 'TTL Slow - 1st Curtain (front)', #13
+            0xab20 => 'TTL Auto - 1st Curtain (front)', #13
+            0xad20 => 'TTL - Red-eye Flash - 1st Curtain (front)', #13
+            0xae20 => 'TTL Slow - Red-eye Flash - 1st Curtain (front)', #13
+            0xaf20 => 'TTL Auto - Red-eye Flash - 1st Curtain (front)', #13
             0xc920 => '2nd Curtain (rear)', #10
+            0xca20 => 'TTL Slow - 2nd Curtain (rear)', #13
+            0xcb20 => 'TTL Auto - 2nd Curtain (rear)', #13
+            0xcd20 => 'TTL - Red-eye Flash - 2nd Curtain (rear)', #13
+            0xce20 => 'TTL Slow - Red-eye Flash - 2nd Curtain (rear)', #13
+            0xcf20 => 'TTL Auto - Red-eye Flash - 2nd Curtain (rear)', #13
             0xe920 => 'High Speed Sync (HSS)', #10
         },
     },
@@ -285,6 +302,7 @@ my %faceCategories = (
         PrintConv => {
             0 => 'Auto',
             1 => 'Manual',
+            65535 => 'Movie', #forum10766
         },
     },
     0x1022 => { #8/forum6579
@@ -450,8 +468,9 @@ my %faceCategories = (
     0x104d => { #forum9634
         Name => 'CropMode',
         Writable => 'int16u',
-        PrintConv => {
+        PrintConv => { # (perhaps this is a bit mask?)
             0 => 'n/a',
+            1 => 'Full-frame on GFX', #IB
             2 => 'Sports Finder Mode', # (mechanical shutter)
             4 => 'Electronic Shutter 1.25x Crop', # (continuous high)
         },
@@ -598,6 +617,7 @@ my %faceCategories = (
             0x501 => 'Pro Neg. Hi', #PH (X-Pro1)
             0x600 => 'Classic Chrome', #forum6109
             0x700 => 'Eterna', #12
+            0x800 => 'Classic Negative', #forum10536
         },
     },
     0x1402 => { #2
@@ -635,7 +655,7 @@ my %faceCategories = (
     },
     # 0x1408 - values: '0100', 'S100', 'VQ10'
     # 0x1409 - values: same as 0x1408
-    # 0x140a - values: 0, 1, 3, 5, 7 (bit 2=red-eye detection, ref 11)
+    # 0x140a - values: 0, 1, 3, 5, 7 (bit 2=red-eye detection, ref 11/13)
     0x140b => { #6
         Name => 'AutoDynamicRange',
         Writable => 'int16u',
@@ -665,6 +685,8 @@ my %faceCategories = (
         PrintConv => {
             0 => 'Unrecognized',
             0x100 => 'Portrait Image',
+            0x103 => 'Night Portrait', #forum10651
+            0x105 => 'Backlit Portrait', #forum10651
             0x200 => 'Landscape Image',
             0x300 => 'Night Scene',
             0x400 => 'Macro',
@@ -709,12 +731,41 @@ my %faceCategories = (
     0x1446 => { #12
         Name => 'FlickerReduction',
         Writable => 'int32u',
+        # seen values: Off=0x0000, On=0x2100,0x3100
         PrintConv => q{
-            my $on = ((($val >> 12) & 0xff) == 3) ? 'On' : 'Off';
+            my $on = ((($val >> 8) & 0x0f) == 1) ? 'On' : 'Off';
             return sprintf('%s (0x%.4x)', $on, $val);
         },
         PrintConvInv => '$val=~/(0x[0-9a-f]+)/i; hex $1',
     },
+    0x3803 => { #forum10037
+        Name => 'VideoRecordingMode',
+        Groups => { 2 => 'Video' },
+        Writable => 'int32u',
+        PrintHex => 1,
+        PrintConv => {
+            0x00 => 'Normal',
+            0x10 => 'F-log',
+            0x20 => 'HLG',
+        },
+    },
+    0x3804 => { #forum10037
+        Name => 'PeripheralLighting',
+        Groups => { 2 => 'Video' },
+        Writable => 'int16u',
+        PrintConv => { 0 => 'Off', 1 => 'On' },
+    },
+    # 0x3805 - int16u: seen 1
+    0x3806 => { #forum10037
+        Name => 'VideoCompression',
+        Groups => { 2 => 'Video' },
+        Writable => 'int16u',
+        PrintConv => {
+            1 => 'Log GOP',
+            2 => 'All Intra',
+        },
+    },
+    # 0x3810 - int32u: related to video codec (ref forum10037)
     0x3820 => { #PH (HS20EXR MOV)
         Name => 'FrameRate',
         Writable => 'int16u',
@@ -730,6 +781,12 @@ my %faceCategories = (
         Writable => 'int16u',
         Groups => { 2 => 'Video' },
     },
+    0x3824 => { #forum10480 (X series)
+        Name => 'FullHDHighSpeedRec',
+        Writable => 'int32u',
+        Groups => { 2 => 'Video' },
+        PrintConv => { 1 => 'Off', 2 => 'On' },
+    },
     0x4005 => { #forum9634
         Name => 'FaceElementSelected', # (could be face or eye)
         Writable => 'int16u',
@@ -840,9 +897,10 @@ my %faceCategories = (
         Name => 'FocusMode2',
         Mask => 0x000000ff,
         PrintConv => {
-            0 => 'AF-M',
-            1 => 'AF-S',
-            2 => 'AF-C',
+            0x00 => 'AF-M',
+            0x01 => 'AF-S',
+            0x02 => 'AF-C',
+            0x11 => 'AF-S (Auto)',
         },
     },
     0.2 => {
@@ -1004,9 +1062,30 @@ my %faceCategories = (
         Groups => { 1 => 'RAF2' }, # (so RAF2 shows up in family 1 list)
         Count => 2,
         Notes => 'including borders',
-        ValueConv => 'my @v=reverse split(" ",$val);"@v"',
+        ValueConv => 'my @v=reverse split(" ",$val);"@v"', # reverse to show width first
         PrintConv => '$val=~tr/ /x/; $val',
     },
+    0x110 => {
+        Name => 'RawImageCropTopLeft',
+        Format => 'int16u',
+        Count => 2,
+        Notes => 'top margin first, then left margin',
+    },
+    0x111 => {
+        Name => 'RawImageCroppedSize',
+        Format => 'int16u',
+        Count => 2,
+        Notes => 'including borders',
+        ValueConv => 'my @v=reverse split(" ",$val);"@v"', # reverse to show width first
+        PrintConv => '$val=~tr/ /x/; $val',
+    },
+    0x115 => {
+        Name => 'RawImageAspectRatio',
+        Format => 'int16u',
+        Count => 2,
+        ValueConv => 'my @v=reverse split(" ",$val);"@v"', # reverse to show width first
+        PrintConv => '$val=~tr/ /:/; $val',
+    },
     0x121 => [
         {
             Name => 'RawImageSize',
@@ -1394,9 +1473,10 @@ sub WriteRAF($$)
         return 1;
     }
     # check to make sure this version of RAF has been tested
-    unless ($testedRAF{$ver}) {
-        $et->Warn("RAF version $ver not yet tested", 1);
-    }
+    #(removed in ExifTool 11.70)
+    #unless ($testedRAF{$ver}) {
+    #    $et->Warn("RAF version $ver not yet tested", 1);
+    #}
     # read the embedded JPEG
     unless ($raf->Seek($jpos, 0) and $raf->Read($jpeg, $jlen) == $jlen) {
         $et->Error('Error reading RAF meta information');
@@ -1559,7 +1639,7 @@ FujiFilm maker notes in EXIF information, and to read/write FujiFilm RAW
 
 =head1 AUTHOR
 
-Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
